@@ -1,12 +1,12 @@
-local t_unpack = table.unpack or unpack; -- luacheck: ignore 113
 local time_now = os.time;
 
-local jid_prep = require "util.jid".prep;
-local set = require "util.set";
-local st = require "util.stanza";
-local it = require "util.iterators";
-local uuid_generate = require "util.uuid".generate;
-local dataform = require"util.dataforms".new;
+local jid_prep = require "prosody.util.jid".prep;
+local set = require "prosody.util.set";
+local st = require "prosody.util.stanza";
+local it = require "prosody.util.iterators";
+local uuid_generate = require "prosody.util.uuid".generate;
+local dataform = require"prosody.util.dataforms".new;
+local errors = require "prosody.util.error";
 
 local xmlns_pubsub = "http://jabber.org/protocol/pubsub";
 local xmlns_pubsub_errors = "http://jabber.org/protocol/pubsub#errors";
@@ -17,7 +17,7 @@ local _M = {};
 local handlers = {};
 _M.handlers = handlers;
 
-local pubsub_errors = {
+local pubsub_errors = errors.init("pubsub", xmlns_pubsub_errors, {
 	["conflict"] = { "cancel", "conflict" };
 	["invalid-jid"] = { "modify", "bad-request", nil, "invalid-jid" };
 	["jid-required"] = { "modify", "bad-request", nil, "jid-required" };
@@ -31,13 +31,14 @@ local pubsub_errors = {
 	["internal-server-error"] = { "wait", "internal-server-error" };
 	["precondition-not-met"] = { "cancel", "conflict", nil, "precondition-not-met" };
 	["invalid-item"] = { "modify", "bad-request", "invalid item" };
-};
-local function pubsub_error_reply(stanza, error)
-	local e = pubsub_errors[error];
-	local reply = st.error_reply(stanza, t_unpack(e, 1, 3));
-	if e[4] then
-		reply:tag(e[4], { xmlns = xmlns_pubsub_errors }):up();
+	["persistent-items-unsupported"] = { "cancel", "feature-not-implemented", nil, "persistent-items" };
+});
+local function pubsub_error_reply(stanza, error, context)
+	local err = pubsub_errors.wrap(error, context);
+	if error == "precondition-not-met" and type(context) == "table" and type(context.field) == "string" then
+		err.text = "Field does not match: " .. context.field;
 	end
+	local reply = st.error_reply(stanza, err);
 	return reply;
 end
 _M.pubsub_error_reply = pubsub_error_reply;
@@ -79,8 +80,9 @@ local node_config_form = dataform {
 	};
 	{
 		type = "text-single";
-		datatype = "xs:integer";
+		datatype = "pubsub:integer-or-max";
 		name = "max_items";
+		range_min = 1;
 		var = "pubsub#max_items";
 		label = "Max # of items to persist";
 	};
@@ -103,6 +105,12 @@ local node_config_form = dataform {
 			"whitelist",
 		};
 	};
+	{
+		type = "list-multi"; -- TODO some way to inject options
+		name = "roster_groups_allowed";
+		var = "pubsub#roster_groups_allowed";
+		label = "Roster groups allowed to subscribe";
+	};
 	{
 		type = "list-single";
 		name = "publish_model";
@@ -114,6 +122,12 @@ local node_config_form = dataform {
 			"open";
 		};
 	};
+	{
+		type = "list-single";
+		var = "pubsub#send_last_published_item";
+		name = "send_last_published_item";
+		options = { "never"; "on_sub"; "on_sub_and_presence" };
+	};
 	{
 		type = "boolean";
 		value = true;
@@ -152,7 +166,19 @@ local node_config_form = dataform {
 		var = "pubsub#notify_retract";
 		value = true;
 	};
+	{
+		type = "list-single";
+		label = "Specify whose JID to include as the publisher of items";
+		name = "itemreply";
+		var = "pubsub#itemreply";
+		options = {
+			{ label = "Include the node owner's JID", value = "owner" };
+			{ label = "Include the item publisher's JID", value = "publisher" };
+			{ label = "Don't include any JID with items", value = "none", default = true };
+		};
+	};
 };
+_M.node_config_form = node_config_form;
 
 local subscribe_options_form = dataform {
 	{
@@ -166,6 +192,7 @@ local subscribe_options_form = dataform {
 		label = "Receive message body in addition to payload?";
 	};
 };
+_M.subscribe_options_form = subscribe_options_form;
 
 local node_metadata_form = dataform {
 	{
@@ -175,17 +202,31 @@ local node_metadata_form = dataform {
 	};
 	{
 		type = "text-single";
-		name = "pubsub#title";
+		name = "title";
+		var = "pubsub#title";
 	};
 	{
 		type = "text-single";
-		name = "pubsub#description";
+		name = "description";
+		var = "pubsub#description";
 	};
 	{
 		type = "text-single";
-		name = "pubsub#type";
+		name = "payload_type";
+		var = "pubsub#type";
+	};
+	{
+		type = "text-single";
+		name = "access_model";
+		var = "pubsub#access_model";
+	};
+	{
+		type = "text-single";
+		name = "publish_model";
+		var = "pubsub#publish_model";
 	};
 };
+_M.node_metadata_form = node_metadata_form;
 
 local service_method_feature_map = {
 	add_subscription = { "subscribe", "subscription-options" };
@@ -237,40 +278,43 @@ function _M.get_feature_set(service)
 		supported_features:add("access-"..service.node_defaults.access_model);
 	end
 
+	if service.node_defaults.send_last_published_item ~= "never" then
+		supported_features:add("last-published");
+	end
+
 	if rawget(service.config, "itemstore") and rawget(service.config, "nodestore") then
 		supported_features:add("persistent-items");
 	end
 
+	if true --[[ node_metadata_form[max_items].datatype == "pubsub:integer-or-max" ]] then
+		supported_features:add("config-node-max");
+	end
+
 	return supported_features;
 end
 
 function _M.handle_disco_info_node(event, service)
 	local stanza, reply, node = event.stanza, event.reply, event.node;
-	local ok, ret = service:get_nodes(stanza.attr.from);
-	local node_obj = ret[node];
-	if not ok or not node_obj then
-		return;
+	local ok, meta = service:get_node_metadata(node, stanza.attr.from);
+	if not ok then
+		event.origin.send(pubsub_error_reply(stanza, meta));
+		return true;
 	end
 	event.exists = true;
 	reply:tag("identity", { category = "pubsub", type = "leaf" }):up();
-	if node_obj.config then
-		reply:add_child(node_metadata_form:form({
-			["pubsub#title"] = node_obj.config.title;
-			["pubsub#description"] = node_obj.config.description;
-			["pubsub#type"] = node_obj.config.payload_type;
-		}, "result"));
-	end
+	reply:add_child(node_metadata_form:form(meta, "result"));
 end
 
 function _M.handle_disco_items_node(event, service)
 	local stanza, reply, node = event.stanza, event.reply, event.node;
 	local ok, ret = service:get_items(node, stanza.attr.from);
 	if not ok then
-		return;
+		event.origin.send(pubsub_error_reply(stanza, ret));
+		return true;
 	end
 
 	for _, id in ipairs(ret) do
-		reply:tag("item", { jid = module.host, name = id }):up();
+		reply:tag("item", { jid = service.config.jid or module.host, name = id }):up();
 	end
 	event.exists = true;
 end
@@ -295,31 +339,59 @@ end
 
 function handlers.get_items(origin, stanza, items, service)
 	local node = items.attr.node;
-	local item = items:get_child("item");
-	local item_id = item and item.attr.id;
+
+	local requested_items = {};
+	for item in items:childtags("item") do
+		table.insert(requested_items, item.attr.id);
+	end
+	if requested_items[1] == nil then
+		requested_items = nil;
+	end
 
 	if not node then
 		origin.send(pubsub_error_reply(stanza, "nodeid-required"));
 		return true;
 	end
-	local ok, results = service:get_items(node, stanza.attr.from, item_id);
+
+	local node_obj = service.nodes[node];
+	if not node_obj then
+		origin.send(pubsub_error_reply(stanza, "item-not-found"));
+		return true;
+	end
+
+	local resultspec; -- TODO rsm.get()
+	if items.attr.max_items then
+		resultspec = { max = tonumber(items.attr.max_items) };
+	end
+	local ok, results = service:get_items(node, stanza.attr.from, requested_items, resultspec);
 	if not ok then
 		origin.send(pubsub_error_reply(stanza, results));
 		return true;
 	end
 
+	local expose_publisher = service.config.expose_publisher;
+	if expose_publisher == nil and node_obj.config.itemreply == "publisher" then
+		expose_publisher = true;
+	end
+
 	local data = st.stanza("items", { node = node });
-	for _, id in ipairs(results) do
-		data:add_child(results[id]);
+	local iter, v, i = ipairs(results);
+	if not requested_items then
+		-- XXX Hack to preserve order of explicitly requested items.
+		iter, v, i = it.reverse(iter, v, i);
 	end
-	local reply;
-	if data then
-		reply = st.reply(stanza)
-			:tag("pubsub", { xmlns = xmlns_pubsub })
-				:add_child(data);
-	else
-		reply = pubsub_error_reply(stanza, "item-not-found");
+
+	for _, id in iter, v, i do
+		local item = results[id];
+		if not expose_publisher then
+			item = st.clone(item);
+			item.attr.publisher = nil;
+		end
+		data:add_child(item);
 	end
+	local reply = st.reply(stanza)
+		:tag("pubsub", { xmlns = xmlns_pubsub })
+			:add_child(data);
 	origin.send(reply);
 	return true;
 end
@@ -490,6 +562,12 @@ function handlers.set_subscribe(origin, stanza, subscribe, service)
 		reply = pubsub_error_reply(stanza, ret);
 	end
 	origin.send(reply);
+	local ok, config = service:get_node_config(node, true);
+	if ok and config.send_last_published_item ~= "never" then
+		local ok, id, item = service:get_last_item(node, jid);
+		if not (ok and id) then return; end
+		service.config.broadcaster("items", node, { [jid] = true }, item);
+	end
 end
 
 function handlers.set_unsubscribe(origin, stanza, unsubscribe, service)
@@ -502,7 +580,13 @@ function handlers.set_unsubscribe(origin, stanza, unsubscribe, service)
 	local ok, ret = service:remove_subscription(node, stanza.attr.from, jid);
 	local reply;
 	if ok then
-		reply = st.reply(stanza);
+		reply = st.reply(stanza)
+			:tag("pubsub", { xmlns = xmlns_pubsub })
+				:tag("subscription", {
+					node = node,
+					jid = jid,
+					subscription = "none"
+				}):up();
 	else
 		reply = pubsub_error_reply(stanza, ret);
 	end
@@ -586,7 +670,10 @@ function handlers.set_publish(origin, stanza, publish, service)
 			item.attr.id = id;
 		end
 	end
-	local ok, ret = service:publish(node, stanza.attr.from, id, item, required_config);
+	if item then
+		item.attr.publisher = service.config.normalize_jid(stanza.attr.from);
+	end
+	local ok, ret, context = service:publish(node, stanza.attr.from, id, item, required_config);
 	local reply;
 	if ok then
 		if type(ok) == "string" then
@@ -597,7 +684,7 @@ function handlers.set_publish(origin, stanza, publish, service)
 				:tag("publish", { node = node })
 					:tag("item", { id = id });
 	else
-		reply = pubsub_error_reply(stanza, ret);
+		reply = pubsub_error_reply(stanza, ret, context);
 	end
 	origin.send(reply);
 	return true;
@@ -606,8 +693,7 @@ end
 function handlers.set_retract(origin, stanza, retract, service)
 	local node, notify = retract.attr.node, retract.attr.notify;
 	notify = (notify == "1") or (notify == "true");
-	local item = retract:get_child("item");
-	local id = item and item.attr.id
+	local id = retract:get_child_attr("item", nil, "id");
 	if not (node and id) then
 		origin.send(pubsub_error_reply(stanza, node and "item-not-found" or "nodeid-required"));
 		return true;
@@ -627,14 +713,13 @@ function handlers.set_retract(origin, stanza, retract, service)
 end
 
 function handlers.owner_set_purge(origin, stanza, purge, service)
-	local node, notify = purge.attr.node, purge.attr.notify;
-	notify = (notify == "1") or (notify == "true");
+	local node = purge.attr.node;
 	local reply;
 	if not node then
 		origin.send(pubsub_error_reply(stanza, "nodeid-required"));
 		return true;
 	end
-	local ok, ret = service:purge(node, stanza.attr.from, notify);
+	local ok, ret = service:purge(node, stanza.attr.from, true);
 	if ok then
 		reply = st.reply(stanza);
 	else
@@ -775,16 +860,15 @@ function handlers.owner_set_affiliations(origin, stanza, affiliations, service)
 	return true;
 end
 
-local function create_encapsulating_item(id, payload)
-	local item = st.stanza("item", { id = id, xmlns = xmlns_pubsub });
+local function create_encapsulating_item(id, payload, publisher)
+	local item = st.stanza("item", { id = id, publisher = publisher, xmlns = xmlns_pubsub });
 	item:add_child(payload);
 	return item;
 end
 
-local function archive_itemstore(archive, config, user, node)
-	module:log("debug", "Creation of itemstore for node %s with config %s", node, config);
+local function archive_itemstore(archive, max_items, user, node)
+	module:log("debug", "Creation of archive itemstore for node %s with limit %d", node, max_items);
 	local get_set = {};
-	local max_items = config["max_items"];
 	function get_set:items() -- luacheck: ignore 212/self
 		local data, err = archive:find(user, {
 			limit = tonumber(max_items);
@@ -795,14 +879,15 @@ local function archive_itemstore(archive, config, user, node)
 			return true;
 		end
 		module:log("debug", "Listed items %s", data);
-		return it.reverse(function()
+		return function()
+			-- luacheck: ignore 211/when
 			local id, payload, when, publisher = data();
 			if id == nil then
 				return;
 			end
 			local item = create_encapsulating_item(id, payload, publisher);
 			return id, item;
-		end);
+		end;
 	end
 	function get_set:get(key) -- luacheck: ignore 212/self
 		local data, err = archive:find(user, {
@@ -857,7 +942,7 @@ local function archive_itemstore(archive, config, user, node)
 			return item.attr.id, item;
 		end
 	end
-	return setmetatable(get_set, archive);
+	return get_set;
 end
 _M.archive_itemstore = archive_itemstore;
 
