@@ -104,6 +104,7 @@ room_mt.get_occupant_id = occupant_id.get_occupant_id;
 local jid_split = require "prosody.util.jid".split;
 local jid_prep = require "prosody.util.jid".prep;
 local jid_bare = require "prosody.util.jid".bare;
+local jid_resource = require "prosody.util.jid".resource;
 local st = require "prosody.util.stanza";
 local cache = require "prosody.util.cache";
 
@@ -441,6 +442,13 @@ module:hook("muc-room-pre-create", function(event)
 	end
 end);
 
+local function is_join_presence(stanza)
+	return stanza.name == "presence"
+		and stanza.attr.type == nil
+		and jid_resource(stanza.attr.to) ~= nil
+		and stanza:get_child("x", "http://jabber.org/protocol/muc");
+end
+
 for event_name, method in pairs {
 	-- Normal room interactions
 	["iq-get/bare/http://jabber.org/protocol/disco#info:query"] = "handle_disco_info_get_query" ;
@@ -496,7 +504,7 @@ for event_name, method in pairs {
 				origin.send(st.error_reply(stanza, "modify", "jid-malformed", nil, module.host));
 				return true;
 			end
-			if stanza.attr.type == nil and stanza.name == "presence" and stanza:get_child("x", "http://jabber.org/protocol/muc") then
+			if is_join_presence(stanza) then
 				room = muclib.new_room(room_jid);
 				return room:handle_first_presence(origin, stanza);
 			elseif stanza.attr.type ~= "error" then
