@@ -313,10 +313,8 @@ static int rb_new(lua_State *L) {
 	return 1;
 }
 
-int luaopen_util_ringbuffer(lua_State *L) {
-#if (LUA_VERSION_NUM > 501)
+int luaopen_prosody_util_ringbuffer(lua_State *L) {
 	luaL_checkversion(L);
-#endif
 
 	if(luaL_newmetatable(L, "ringbuffer_mt")) {
 		lua_pushcfunction(L, rb_tostring);
@@ -355,3 +353,7 @@ int luaopen_util_ringbuffer(lua_State *L) {
 	lua_setfield(L, -2, "new");
 	return 1;
 }
+
+int luaopen_util_ringbuffer(lua_State *L) {
+	return luaopen_prosody_util_ringbuffer(L);
+}
