@@ -1,15 +1,16 @@
 -- Prosody IM
--- Copyright (C) 2008-2009 Matthew Wild
--- Copyright (C) 2008-2009 Waqas Hussain
--- 
+-- Copyright (C) 2008-2010 Matthew Wild
+-- Copyright (C) 2008-2010 Waqas Hussain
+--
 -- This project is MIT/X11 licensed. Please see the
 -- COPYING file in the source package for more information.
 --
 
 
 
+local unpack = table.unpack;
 local t_insert = table.insert;
-function import(module, ...)
+function _G.import(module, ...)
 	local m = package.loaded[module] or require(module);
 	if type(m) == "table" and ... then
 		local ret = {};
